/*
 * davinci_elno  --  SoC audio for ELNO board
 *
 *  Copyright (C) 2012 ELNO
 *
 * Authors: Cyril GERMAINE
 *
 */

#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/kernel.h>
#include <linux/clk.h>
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/platform_device.h>
#include <linux/i2c.h>

#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/pcm_params.h>
#include <sound/soc.h>
#include <sound/soc-dapm.h>

#include <asm/mach-types.h>
#include <mach/hardware.h>
#include <mach/gpio.h>

#include "../codecs/wm8731.h"
#include "davinci-pcm.h"
#include "davinci-i2s.h"
#include "davinci-mcasp.h"

#define MCLK_RATE 12000000


static int davinci_elno_hw_params(struct snd_pcm_substream *substream,
	struct snd_pcm_hw_params *params)
{
	struct snd_soc_pcm_runtime *rtd = substream->private_data;
	struct snd_soc_dai *codec_dai = rtd->codec_dai;
	struct snd_soc_dai *cpu_dai = rtd->cpu_dai;
	int ret;

	/* set codec DAI configuration */
	ret = snd_soc_dai_set_fmt(codec_dai, SND_SOC_DAIFMT_I2S |
		SND_SOC_DAIFMT_NB_NF | SND_SOC_DAIFMT_CBM_CFM);
	if (ret < 0)
		return ret;

	/* set cpu DAI configuration */
	ret = snd_soc_dai_set_fmt(cpu_dai, SND_SOC_DAIFMT_I2S |
		SND_SOC_DAIFMT_NB_NF | SND_SOC_DAIFMT_CBM_CFM);
	if (ret < 0)
		return ret;


	return 0;
}

static struct snd_soc_ops davinci_elno_ops = {
	.hw_params = davinci_elno_hw_params,
};

static const struct snd_soc_dapm_widget davinci_elno_dapm_widgets[] = {
	SND_SOC_DAPM_MIC("Int Mic", NULL),
	SND_SOC_DAPM_SPK("Ext Spk", NULL),
	SND_SOC_DAPM_LINE("Line In", NULL),
};

static const struct snd_soc_dapm_route intercon[] = {
	{"LLINEIN", NULL, "Line In"},
	{"RLINEIN", NULL, "Line In"},
};

/*
 * Logic for a wm8731 as connected on a davinci_elno board.
 */
static int davinci_elno_wm8731_init(struct snd_soc_pcm_runtime *rtd)
{
	struct snd_soc_codec *codec = rtd->codec;
	struct snd_soc_dapm_context *dapm = &codec->dapm;
	struct snd_soc_dai *codec_dai = rtd->codec_dai;
	int ret;

	printk("ASoC: davinci_elno_wm8731_init\n");

	printk(KERN_DEBUG
			"davinci_elno_wm8731 "
			": davinci_elno_wm8731_init() called\n");

	ret = snd_soc_dai_set_sysclk(codec_dai, WM8731_SYSCLK_MCLK,
		MCLK_RATE, SND_SOC_CLOCK_IN);
	if (ret < 0) {
		printk(KERN_ERR "Failed to set WM8731 SYSCLK: %d\n", ret);
		return ret;
	}

	/* Add specific widgets */
	snd_soc_dapm_new_controls(dapm, davinci_elno_dapm_widgets,
				  ARRAY_SIZE(davinci_elno_dapm_widgets));
	/* Set up specific audio path interconnects */
	snd_soc_dapm_add_routes(dapm, intercon, ARRAY_SIZE(intercon));

	/* always connected */
	snd_soc_dapm_enable_pin(dapm, "RLINEIN");
	snd_soc_dapm_enable_pin(dapm, "LLINEIN");

	/* not connected */
	snd_soc_dapm_nc_pin(dapm, "Int Mic");
	snd_soc_dapm_nc_pin(dapm, "Ext Spk");

	snd_soc_dapm_sync(dapm);

	return 0;
}

static struct snd_soc_dai_link davinci_elno_dai = {
	.name = "WM8731",
	.stream_name = "WM8731 PCM",
	.codec_dai_name = "wm8731-hifi",
	.cpu_dai_name = "davinci-mcbsp",
	.init = davinci_elno_wm8731_init,
	.ops = &davinci_elno_ops,
	.platform_name = "davinci-pcm-audio",
	.codec_name = "wm8731.1-001a",

};

static struct snd_soc_card snd_soc_davinci_elno = {
	.name = "DAVINCI-ELNO AUDIO SOC",
	.owner = THIS_MODULE,
	.dai_link = &davinci_elno_dai,
	.num_links = 1,
};

static int __devinit davinci_elno_probe(struct platform_device *pdev)
{
	int ret;
	struct snd_soc_card *card = &snd_soc_davinci_elno;

	printk("ASoC: davinci_elno_probe\n");

	card->dev = &pdev->dev;
	ret = snd_soc_register_card(card);


	if (ret)
	{
		dev_err(&pdev->dev, "snd_soc_register_card() failed:%d\n", ret);
	}

	return ret;
}

static int __devexit davinci_elno_remove(struct platform_device *pdev)
{

	struct snd_soc_card *card = platform_get_drvdata(pdev);

	snd_soc_unregister_card(card);

	return 0;

}

static struct platform_driver davinci_elno_audio_driver = {
	.driver		= {
		.name	= "davinci-elno-audio",
		.owner	= THIS_MODULE,
	},
	.probe		= davinci_elno_probe,
	.remove		= __devexit_p(davinci_elno_remove),
};
module_platform_driver(davinci_elno_audio_driver);


/* Module information */
MODULE_AUTHOR("Cyril GERMAINE");
MODULE_DESCRIPTION("ALSA SoC DAVINCI_ELNO");
MODULE_LICENSE("");
MODULE_ALIAS("platform:davinci-elno-audio");

